? zaptel-mon.diff
Index: ztmonitor.c
===================================================================
RCS file: /usr/cvsroot/zaptel/ztmonitor.c,v
retrieving revision 1.9
diff -u -r1.9 ztmonitor.c
--- ztmonitor.c	4 Nov 2004 03:53:37 -0000	1.9
+++ ztmonitor.c	26 Jul 2005 14:16:34 -0000
@@ -261,11 +261,12 @@
 	char  output_file[255];
 	int res, res2;
 	int visual = 0;
+	int use_audio = 0;
 	int x,i;
 	struct zt_confinfo zc;
 
 	if ((argc < 2) || (atoi(argv[1]) < 1)) {
-		fprintf(stderr, "Usage: ztmonitor <channel num> [-v[v]] [-f FILE]\n");
+		fprintf(stderr, "Usage: ztmonitor <channel num> [-v[v]] [-s] [-a] [-f FILE]\n");
 		exit(1);
 	}
 	for (i = 2; i < argc; ++i) {
@@ -276,8 +277,12 @@
 		} else if (!strcmp(argv[i], "-vv")) {
 			visual = 1;
 			verbose = 1;
+		} else if (!strcmp(argv[i], "-s")) {
+			stereo = 1;
+		} else if (!strcmp(argv[i], "-a")) {
+			use_audio = 1;
        	} else if (!strcmp(argv[i], "-f") && (i+1) < argc) {
-			++i; /*we care about hte file name */
+			++i; /* we care about the file name */
 			if (strlen(argv[i]) < 255 ) {
 				strcpy(output_file, argv[i]);
 				fprintf(stderr, "Output to %s\n", output_file);
@@ -285,13 +290,15 @@
 					fprintf(stderr, "Could not open %s for writing: %s\n", output_file, strerror(errno));
 					exit(0);
 				}
-				fprintf(stderr, "Run e.g., 'sox -r 8000 -s -w -c 1 file.raw file.wav' to convert.\n");
 			} else {
 				fprintf(stderr, "File Name %s too long\n",argv[i+1]);
 			}
 		}
 	}
-	if (!visual) {
+	if (ofh != (FILE *)NULL) {
+		fprintf(stderr, "Run e.g., 'sox -r 8000 -s -w -c %d file.raw file.wav' to convert.\n", stereo ? 2 : 1);
+	}
+	if (!visual || use_audio) {
 		/* Open audio */
 		if ((afd = audio_open()) < 0) {
 			printf("Cannot open audio ...\n");
@@ -301,13 +308,13 @@
 	/* Open Pseudo device */
 	if ((pfd = pseudo_open()) < 0)
 		exit(1);
-	if (visual && ((pfd2 = pseudo_open()) < 0))
+	if ((visual || stereo) && ((pfd2 = pseudo_open()) < 0))
 		exit(1);
 	/* Conference them */
 	memset(&zc, 0, sizeof(zc));
 	zc.chan = 0;
 	zc.confno = atoi(argv[1]);
-	if (visual) {
+	if (visual || stereo) {
 		/* Two pseudo's, one for tx, one for rx */
 		zc.confmode = ZT_CONF_MONITORTX;
 		if (ioctl(pfd, ZT_SETCONF, &zc) < 0) {
@@ -336,33 +343,47 @@
 		printf("( # = Audio Level  * = Max Audio Hit )\n");
 		draw_barheader();
 	}
+	/* Switch stdin into non-blocking mode */
+	res = fcntl(0, F_GETFL, 0);
+	res |= O_NDELAY;
+	fcntl(0, F_SETFL, res);
 	/* Now, copy from pseudo to audio */
 	for (;;) {
+		res = read(0, buf, sizeof(buf));
+		if ((res > 0) || ((res < 0) && (errno != EAGAIN)))
+			break;
 		res = read(pfd, buf, sizeof(buf));
 		if (res < 1) 
 			break;
+		res2 = read(pfd2, buf2, res);
+		if (res2 < 1) 
+			break;
+		if (res != res2) {
+			printf("Huh?  res = %d, res2 = %d?\n", res, res2);
+			break;
+		}
 		if (visual) {
-			res2 = read(pfd2, buf2, res);
-			if (res2 < 1) 
-				break;
-			if (res == res2)
 				visualize((short *)buf, (short *)buf2, res/2);
-			else
-				printf("Huh?  res = %d, res2 = %d?\n", res, res2);
-			
-		} else {
-			if (ofh)	        
-				fwrite(buf, 1, res, ofh);
-		 	if (afd) {
-				if (stereo) {
-					for (x=0;x<res;x++)
-						buf2[x<<1] = buf2[(x<<1) + 1] = buf[x];
-					write(afd, buf2, res << 1);
-				} else
-					write(afd, buf, res);
+		} /* else */ {
+			if (stereo) {
+				/* Make buf2 for stereo */
+				for (x=res;x-- > 0; ) {
+					buf2[(x<<1) + 1] = buf2[x];
+					buf2[x<<1] = buf[x];
+				}
+				res <<= 1;
+			} else {
+				/* Merge TX and RX */
+				for(x=0; x < res; ++x)
+					buf2[x] = (short)(((int)buf2[x<<1] + (int)buf2[(x<<1) + 1]) >> 1);
 			}
+			if (ofh)
+				fwrite(buf2, 1, res, ofh);
+		 	if (afd >= 0)
+				write(afd, buf2, res);
 		}
 	}
-	if (ofh) fclose(ofh); /*Never Reached */
+	if (ofh)
+		fclose(ofh); /*Never Reached */
 	exit(0);
 }
